import * as server from '@minecraft/server'

import * as ui from '@minecraft/server-ui'

import { maxWingedStamina } from './main_pik'

const OWNER_NAME_PREFIX = "§dOwner: §e"

const world = server.world
const system = server.system

export {OWNER_NAME_PREFIX}

export function getScore(objective, target, useZero = false) {
    try {
        const oB = world.scoreboard.getObjective(objective)
        if (typeof target == 'string') return oB.getScore(oB.getParticipants().find(pT => pT.displayName == target))
        const r = oB.getScore(target.scoreboardIdentity)
        return r
    } catch {
        return 0
    }
  }
export function onionInterface(/**@type {import("@minecraft/server").Player}*/ player,/**@type {import("@minecraft/server").Entity}*/ onion){
    const color = onion.typeId.substring(0,onion.typeId.indexOf("onion"))+"pikmin_item"
    const pInventory = player.getComponent('inventory').container
    const hpik = world.scoreboard.getObjective("hpik")
  
    var pikOnPlayer = 0
    //Counts the amount of pikmin of the requiered type
    
    for (let i = 0; i < player.getComponent('inventory').container.size; i++) {
      var item = pInventory.getItem(i)
      if(item && item.typeId==color){
        pikOnPlayer += item.amount
      }
    }
  
  
  
    var pikOnOnion = getScore("hpik",onion)
    
  
    const form = new ui.ModalFormData()
    form.title({"rawtext":[{"translate":"pik.onion_menu.title"}]})
    
    form.toggle({"rawtext":[{"translate":"pik.onion_menu.exit_toggle"}]},(onion.getComponent("minecraft:is_chested")==undefined))
    
    form.slider({"rawtext":[{"translate":"pik.onion_menu.deposit"}]},0,pikOnPlayer,1,0)
  
    form.slider({"rawtext":[{"translate":"pik.onion_menu.withdraw"}]},0,pikOnOnion,1,0)
  
    form.show(player).then(onFulfilled =>{
      if(!onFulfilled.canceled){
        let deposit = onFulfilled.formValues[1]
  
        let withdraw = onFulfilled.formValues[2]
  
        let gain = withdraw - deposit//Calculates the gain of pikmin for the player
      
        if(gain>0){//If the player recives pikmin
          pikOnOnion = getScore("hpik",onion)//Updates the amount of pikmin i  the onion
          
          if(pikOnOnion<gain){
            gain = getScore("hpik",onion)//Corrects the gain if needed
          }
          
          hpik.setScore(onion.scoreboardIdentity,pikOnOnion-gain)
          player.runCommandAsync(`/give @s ${color} ${gain} 0 {\"minecraft:keep_on_death\":{}}`)
        }else if(gain<0){
          pikOnOnion = getScore("hpik",onion)//Updates the amount of pikmin i  the onion
  
          pikOnPlayer = 0
  
          for (let i = 0; i < player.getComponent('inventory').container.size; i++) {
            var item = pInventory.getItem(i)
            if(item && item.typeId==color){
              pikOnPlayer += item.amount
            }
          }
  
          if(pikOnPlayer<Math.abs(gain)){
            gain = pikOnPlayer
          }else{
            gain = Math.abs(gain)
          }
          hpik.setScore(onion.scoreboardIdentity,pikOnOnion+gain)
          player.runCommandAsync(`/clear @s ${color} 0 ${gain}`)
  
  
        }
        
        if(onFulfilled.formValues[0]){
          //onion.triggerEvent("let_pikmin_go_outside")
          onion.runCommandAsync("/event entity @s let_pikmin_go_outside")
  
        }else{
          //onion.triggerEvent("keep_pikmin_inside")
          onion.runCommandAsync("/event entity @s keep_pikmin_inside")
        }
        
      }
      
    })
  
  }
export function pikSpawnSeeds(onion, n, altitude){
  let seedId = ""
  if(!onion.typeId){
    console.warn("The entity doesnt has a valid id")
    return
  }else if(onion.typeId.includes("candypop_bud")){
    seedId = onion.typeId.substring(0,onion.typeId.indexOf("candypop_bud"))+"pikmin_seed"
  }else if(onion.typeId.includes("onion")){
    seedId = onion.typeId.substring(0,onion.typeId.indexOf("onion"))+"pikmin_seed"
  }else{
    console.warn(`${onion.typeId} is not a valid option for generating seeds`)
    return
  }
  const {location, dimension} = onion
  const {x: px,y: py,z: pz} = location
  for (let i = 0; i < n; i++) {

      
      spawnSeedDelayed(dimension,seedId,{x: px, y: py+altitude, z: pz})

    
  }


}
function spawnSeedDelayed(dimension,seedId,position){
  system.runTimeout(() => {
    dimension.spawnEntity(seedId,position)
  }, 5 * Math.random());
}
export function yellowPikminSpark(pikmin){
  const {location, dimension} = pikmin
  let ownerName = ""

  if(pikmin.nameTag.includes(OWNER_NAME_PREFIX)){
    ownerName = pikmin.nameTag.replace(OWNER_NAME_PREFIX,``)
  }else{
    ownerName = pikmin.nameTag="patata"
  }
  
  //const empty = new world.MolangVariableMap()
  dimension.spawnParticle(`pik:spark`,location,new server.MolangVariableMap())
  world.playSound(`pikmin.spark`,location)
  dimension.getEntities({location, excludeTypes:[`minecraft:item`], maxDistance: 3, excludeFamilies:[`pikmin`,`inanimate`],excludeNames:[ownerName]}).forEach(entity => {
    if(!entity.hasComponent(`minecraft:fire_immune`) && !entity.getEffect(`fire_resistance`))
    entity.setOnFire(3,true)
    entity.applyDamage(2,{cause: `lightning`, damagingEntity: pikmin})
    if(entity.typeId === `minecraft:creeper`){
      entity.triggerEvent(`minecraft:become_charged`)
    }else if(entity.typeId === `minecraft:villager_v2`){
      entity.triggerEvent(`become_witch`)
    }else if(entity.typeId === `minecraft:pig`){
      entity.triggerEvent(`become_zombie`)
    }
    entity.clearVelocity()
  })
}
export function purplePikminThud(pikmin){
  const {location, dimension} = pikmin
  let ownerName = ""
  let applySlowness = false
  if(Math.random()<=0.4){
    applySlowness = true
  }

  if(pikmin.nameTag.includes(OWNER_NAME_PREFIX)){
    ownerName = pikmin.nameTag.replace(OWNER_NAME_PREFIX,``)
  }else{
    ownerName = pikmin.nameTag="patata"
  }
  
  world.playSound(`pikmin.purple.thud`,location)
  
  dimension.getEntities({location,excludeTypes:[`minecraft:item`], maxDistance: 3, excludeFamilies:[`pikmin`,`inanimate`],excludeNames:[ownerName]}).forEach(entity => {
    entity.applyDamage(2,{cause: `contact`, damagingEntity: pikmin})
    if(entity.typeId!=`minecraft:ender_dragon`){
      entity.applyDamage(2,{cause: `contact`, damagingEntity: pikmin})
      entity.applyDamage(2,{cause: `suicide`, damagingEntity: pikmin})
    }else{
      entity.runCommandAsync(`/damage @s 4 entity_attack entity @p`)
    }
    
    if(applySlowness){
      entity.addEffect("slowness",40,{amplifier:255})
      entity.addEffect("weakness",40,{amplifier:255})
    }
    if(entity.typeId !=`minecraft:player`){entity.clearVelocity()}
    
  })
  
}

export function pikminGrabDamage(pikmin){
  const {target,typeId} = pikmin
  if(!typeId.includes("yellow")){
    let damage = undefined
    if (typeId.includes("purple") || typeId.includes("red")){
      damage = 3
    }else if (typeId.includes("white") || typeId.includes("winged")){
      damage = 1
    }else{
      damage = 2
    }
    if(typeId.includes("white")){
      target.addEffect("wither",100,{amplifier:2})
    }
    if(pikmin.hasComponent(`minecraft:is_stunned`)){
      damage = damage*2
    }
    if(target.typeId!=`minecraft:ender_dragon`){
      if(Math.random()>0.5){
        target.applyDamage(damage,{cause: `entityAttack`, damagingEntity: pikmin})

      }else{
        target.applyDamage(damage,{cause: `suicide`, damagingEntity: pikmin})
      }
    }else{
      
      target.runCommandAsync(`/damage @s ${damage} entity_attack entity @p`)
    }
   
    target.clearVelocity()
  }else{
    yellowPikminSpark(pikmin)
  }
}

export function setPikminGrabCordinates(pikmin){
  const obX = world.scoreboard.getObjective(`pGrabX`)
  const obY = world.scoreboard.getObjective(`pGrabY`)
  const obZ = world.scoreboard.getObjective(`pGrabZ`)

  const {target} = pikmin
  const {location:lTarget} = target
  const {location:lPikmin} = pikmin
  const x = lPikmin.x - lTarget.x
  const y = lPikmin.y - lTarget.y
  const z = lPikmin.z - lTarget.z
  obX.setScore(pikmin,Math.floor(x*10))
  obY.setScore(pikmin,Math.floor(y*10))
  obZ.setScore(pikmin,Math.floor(z*10))

  
}




export function pikSpawnNectar(entity){
  const {dimension, location } = entity

  const health = Math.floor(entity.getComponent("minecraft:health").effectiveMax)

  const cNectar = Math.floor(health/16)
  const smallCNectar = Math.floor((health%16)/4)
  if (cNectar > 0){
    const toSpawn = new server.ItemStack("pik:crystalized_nectar",cNectar)
    dimension.spawnItem(toSpawn,location)
  }
  if (smallCNectar > 0){
    const toSpawn = new server.ItemStack("pik:small_crystalized_nectar",smallCNectar)
    dimension.spawnItem(toSpawn,location)
  }
}

export function wingedPikminCarryPlayer(player,mainHand,offHand){

  const wingedStamina = player.getDynamicProperty(`wingedStamina`)
  let wingedOnMain = (mainHand && mainHand.typeId === `pik:winged_pikmin_item`)
  let wingedOnOff = (offHand && offHand.typeId === `pik:winged_pikmin_item`)
  if(( wingedOnMain || wingedOnOff ) && !(player.isOnGround) && !player.isSneaking){
    player.addEffect(`slow_falling`,5,{showParticles:false})
    
    if(!wingedOnOff && wingedOnMain){
      player.addEffect(`weakness`,5,{showParticles:false,amplifier:255})
      
        player.playAnimation(`animation.carried_by_winged_main`, {stopExpression: `query.is_on_ground || (!query.is_item_name_any('slot.weapon.mainhand', 0, 'pik:winged_pikmin_item')) || query.is_sneaking`})
        
    }
    if(wingedOnOff && !wingedOnMain){
      
        player.playAnimation(`animation.carried_by_winged_off`, {stopExpression: `query.is_on_ground || (!query.is_item_name_any('slot.weapon.offhand', 0, 'pik:winged_pikmin_item')) || query.is_sneaking`})
        
    }
    if(wingedOnOff && wingedOnMain){
      player.addEffect(`weakness`,5,{showParticles:false,amplifier:255})
      if(player.isJumping && (wingedStamina > 0) && !player.hasTag(`wingedExhausted`)){
        player.addEffect(`levitation`,5,{showParticles:false})
        player.setDynamicProperty(`wingedStamina`,wingedStamina-1)
        if(wingedStamina-1 == 0 && !player.hasTag(`wingedExhausted`)){player.addTag(`wingedExhausted`)}
      }
        
        player.playAnimation(`animation.carried_by_winged`, {stopExpression: `query.is_on_ground || (!query.is_item_name_any('slot.weapon.mainhand', 0, 'pik:winged_pikmin_item')) || !query.is_item_name_any('slot.weapon.offhand', 0, 'pik:winged_pikmin_item') || query.is_sneaking`})
        
    }
    
  }else if(player.isOnGround){
    
    if((wingedStamina+5)>maxWingedStamina){
      player.setDynamicProperty(`wingedStamina`,maxWingedStamina)
      if(player.hasTag(`wingedExhausted`)){player.removeTag(`wingedExhausted`)}
      
    }else{
      player.setDynamicProperty(`wingedStamina`,wingedStamina+5)
    }
  }
  if(wingedShouldTellStamina(player)){
    if(player.hasTag(`wingedExhausted`)){
      player.onScreenDisplay.setActionBar( {"rawtext":[{"translate":`pik.wingedStamina.indicator_exhausted`,"with":[`${Math.floor((wingedStamina/maxWingedStamina)*100)}`]}]})
    }else{
      player.onScreenDisplay.setActionBar( {"rawtext":[{"translate":`pik.wingedStamina.indicator`,"with":[`${Math.floor((wingedStamina/maxWingedStamina)*100)}`]}]})
    }
    
    //console.warn(wingedStamina)
  }
 
}

export function wingedShouldTellStamina(player){
  const mainHand =player.getComponent("equippable").getEquipment(server.EquipmentSlot.Mainhand)
  const offHand =player.getComponent("equippable").getEquipment(server.EquipmentSlot.Offhand)
  let wingedOnMain = (mainHand && mainHand.typeId === `pik:winged_pikmin_item`)
  let wingedOnOff = (offHand && offHand.typeId === `pik:winged_pikmin_item`)
  return ( wingedOnMain || wingedOnOff ) && (player.getDynamicProperty(`wingedStamina`) < maxWingedStamina)
}

export function esfericasACartesianas(magnitud, theta, phi) {
  const x = magnitud * Math.sin(phi) * Math.cos(theta);
  const y = magnitud * Math.sin(phi) * Math.sin(theta);
  const z = magnitud * Math.cos(phi);
  return { x, y, z };
}